using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace C1CustomDropDown
{
	/// <summary>
	/// Summary description for CountryDropDown.
	/// </summary>
	public class CountryDropDown : C1.Win.C1Input.DropDownForm
	{
		private System.Windows.Forms.CheckedListBox checkedListBox1;
		private static bool recursion = false;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public CountryDropDown()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.checkedListBox1 = new System.Windows.Forms.CheckedListBox();
			this.SuspendLayout();
			// 
			// checkedListBox1
			// 
			this.checkedListBox1.CheckOnClick = true;
			this.checkedListBox1.ColumnWidth = 90;
			this.checkedListBox1.Dock = System.Windows.Forms.DockStyle.Fill;
			this.checkedListBox1.IntegralHeight = false;
			this.checkedListBox1.Location = new System.Drawing.Point(0, 0);
			this.checkedListBox1.MultiColumn = true;
			this.checkedListBox1.Name = "checkedListBox1";
			this.checkedListBox1.Size = new System.Drawing.Size(300, 172);
			this.checkedListBox1.TabIndex = 0;
			this.checkedListBox1.ThreeDCheckBoxes = true;
			this.checkedListBox1.ItemCheck += new System.Windows.Forms.ItemCheckEventHandler(this.checkedListBox1_ItemCheck);
			// 
			// CountryDropDown
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 14);
			this.ClientSize = new System.Drawing.Size(300, 172);
			this.Controls.Add(this.checkedListBox1);
			this.FocusControl = this.checkedListBox1;
			this.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.Name = "CountryDropDown";
			this.Options = ((C1.Win.C1Input.DropDownFormOptionsFlags)(((C1.Win.C1Input.DropDownFormOptionsFlags.Focusable | C1.Win.C1Input.DropDownFormOptionsFlags.AlwaysPostChanges) 
				| C1.Win.C1Input.DropDownFormOptionsFlags.AutoResize)));
			this.Load += new System.EventHandler(this.CountryDropDown_Load);
			this.VisibleChanged += new System.EventHandler(this.CountryDropDown_VisibleChanged);
			this.PostChanges += new System.EventHandler(this.CountryDropDown_PostChanges);
			this.ResumeLayout(false);

		}
		#endregion

		// Load country names into the CheckedListBox control
		private void CountryDropDown_Load(object sender, System.EventArgs e)
		{
			Form1 f1 = this.Owner as Form1;
			CountriesDataSet ds = f1.countriesDataSet;
	
			// Add an item for checking/unchecking the entire list
			checkedListBox1.Items.Add("(All)");

			for (int i = 0; i < ds.Countries.Count; i++)
			{
				checkedListBox1.Items.Add(ds.Countries[i].Country);
			}
		}

		// Apply a check box state to all items
		private void CheckAllItems(CheckState value)
		{
			for (int i = 0; i < checkedListBox1.Items.Count; i++)
			{
				checkedListBox1.SetItemCheckState(i, value);
			}
		}

		// Fired when the user checks/unchecks an item, but before the value itself changes
		private void checkedListBox1_ItemCheck(object sender, System.Windows.Forms.ItemCheckEventArgs e)
		{
			// Recursion breaker (calling SetItemCheckState will re-fire this event)
			if (recursion)
				return;

			recursion = true;

			// If the "(All)" item is clicked, apply the new value to all items
			if (e.Index == 0)
				CheckAllItems(e.NewValue);
			
			// If any item is unchecked, then uncheck the "all" item
			else if (e.NewValue == CheckState.Unchecked)
				checkedListBox1.SetItemCheckState(0, e.NewValue);

			recursion = false;
		}

		// Set the text of the parent control when the user closes the dropdown
		private void CountryDropDown_PostChanges(object sender, System.EventArgs e)
		{
			// If "(All)" is checked, clear the text in the parent
			if (checkedListBox1.GetItemChecked(0))
			{
				this.OwnerControl.Text = "";
				return;
			}

			StringBuilder sb = new StringBuilder();

			// Build a semicolon delimited string of the selected item
			for (int i = 0; i < checkedListBox1.CheckedItems.Count; i++)
			{
				if (sb.Length > 0)
					sb.Append(";");

				sb.Append(checkedListBox1.CheckedItems[i].ToString());
			}

			this.OwnerControl.Text = sb.ToString();
		}

		private void CountryDropDown_VisibleChanged(object sender, System.EventArgs e)
		{
			// Do nothing if the dropdown is being closed
			if (!this.Visible)
				return;

			string value = this.OwnerControl.Text;
			
			// If the parent control's text is "(All)", treat it as an empty string
			if (value == checkedListBox1.Items[0].ToString())
				value = "";
			
			// Prevent the ItemCheck event from firing now
			recursion = true;
			
			// Clear all items before checking individual countries, otherwise check them all
			CheckAllItems(value.Length > 0 ? CheckState.Unchecked : CheckState.Checked);

			if (value.Length > 0)
			{
				// The parent control's text is a semicolon delimited string of country names
				string[] countries = value.Split(";".ToCharArray());

				for (int i = 0; i < countries.Length; i++)
				{
					int n = checkedListBox1.FindStringExact(countries[i]);

					// Check the item, if found
					if (n != -1)
						checkedListBox1.SetItemCheckState(n, CheckState.Checked);
				}
			}

			recursion = false;
		}
	}
}
